@@ -20,10 +20,15 @@ from __future__ import print_function
 
 import six
 
+from tensorflow.python.feature_column import dense_features
+from tensorflow.python.feature_column import dense_features_v2
 from tensorflow.python.feature_column import feature_column
 from tensorflow.python.feature_column import feature_column_lib
 from tensorflow.python.framework import ops
 from tensorflow.python.keras.engine import training
+from tensorflow.python.keras.layers import core as keras_core
+from tensorflow.python.keras.layers import normalization_v2 as keras_norm
+from tensorflow.python.keras.utils import losses_utils
 from tensorflow.python.layers import core as core_layers
 from tensorflow.python.layers import normalization
 from tensorflow.python.ops import init_ops
@@ -32,13 +37,14 @@ from tensorflow.python.ops import partitioned_variables
 from tensorflow.python.ops import variable_scope
 from tensorflow.python.ops.losses import losses
 from tensorflow.python.summary import summary
+from tensorflow.python.training import training_util
 from tensorflow.python.util.tf_export import estimator_export
 from tensorflow_estimator.python.estimator import estimator
-from tensorflow_estimator.python.estimator import model_fn
 from tensorflow_estimator.python.estimator.canned import head as head_lib
 from tensorflow_estimator.python.estimator.canned import optimizers
 from tensorflow_estimator.python.estimator.head import head_utils
 from tensorflow_estimator.python.estimator.head import regression_head
+from tensorflow_estimator.python.estimator.mode_keys import ModeKeys
 
 # The default learning rate of 0.05 is a historical artifact of the initial
 # implementation, but seems a reasonable choice.
@@ -50,7 +56,7 @@ def _add_hidden_layer_summary(value, tag):
   summary.histogram('%s/activation' % tag, value)
 
 
-@estimator_export('estimator.experimental.dnn_logit_fn_builder')
+@estimator_export(v1=['estimator.experimental.dnn_logit_fn_builder'])
 def dnn_logit_fn_builder(units, hidden_units, feature_columns, activation_fn,
                          dropout, input_layer_partitioner, batch_norm):
   """Function builder for a dnn logit_fn.
@@ -73,7 +79,7 @@ def dnn_logit_fn_builder(units, hidden_units, feature_columns, activation_fn,
   Raises:
     ValueError: If units is not an int.
   """
-  if not isinstance(units, int):
+  if not isinstance(units, six.integer_types):
     raise ValueError('units must be an int.  Given type: {}'.format(
         type(units)))
 
@@ -105,6 +111,57 @@ def dnn_logit_fn_builder(units, hidden_units, feature_columns, activation_fn,
   return dnn_logit_fn
 
 
+def dnn_logit_fn_builder_v2(units, hidden_units, feature_columns, activation_fn,
+                            dropout, batch_norm):
+  """Function builder for a dnn logit_fn.
+
+  Args:
+    units: An int indicating the dimension of the logit layer.  In the MultiHead
+      case, this should be the sum of all component Heads' logit dimensions.
+    hidden_units: Iterable of integer number of hidden units per layer.
+    feature_columns: Iterable of `feature_column._FeatureColumn` model inputs.
+    activation_fn: Activation function applied to each layer.
+    dropout: When not `None`, the probability we will drop out a given
+      coordinate.
+    batch_norm: Whether to use batch normalization after each hidden layer.
+
+  Returns:
+    A logit_fn (see below).
+
+  Raises:
+    ValueError: If units is not an int.
+  """
+  if not isinstance(units, six.integer_types):
+    raise ValueError('units must be an int.  Given type: {}'.format(
+        type(units)))
+
+  def dnn_logit_fn(features, mode):
+    """Deep Neural Network logit_fn.
+
+    Args:
+      features: This is the first item returned from the `input_fn` passed to
+        `train`, `evaluate`, and `predict`. This should be a single `Tensor` or
+        `dict` of same.
+      mode: Optional. Specifies if this training, evaluation or prediction. See
+        `ModeKeys`.
+
+    Returns:
+      A `Tensor` representing the logits, or a list of `Tensor`'s representing
+      multiple logits in the MultiHead case.
+    """
+    dnn_model = _DNNModelV2(
+        units,
+        hidden_units,
+        feature_columns,
+        activation_fn,
+        dropout,
+        batch_norm,
+        name='dnn')
+    return dnn_model(features, mode)
+
+  return dnn_logit_fn
+
+
 def _get_previous_name_scope():
   current_name_scope = ops.get_name_scope()
   return current_name_scope.rsplit('/', 1)[0] + '/'
@@ -125,7 +182,7 @@ class _DNNModel(training.Model):
                **kwargs):
     super(_DNNModel, self).__init__(name=name, **kwargs)
     if feature_column_lib.is_feature_column_v2(feature_columns):
-      self._input_layer = feature_column_lib.DenseFeatures(
+      self._input_layer = dense_features.DenseFeatures(
           feature_columns=feature_columns, name='input_layer')
     else:
       self._input_layer = feature_column.InputLayer(
@@ -182,7 +239,7 @@ class _DNNModel(training.Model):
     self._input_layer_partitioner = input_layer_partitioner
 
   def call(self, features, mode):
-    is_training = mode == model_fn.ModeKeys.TRAIN
+    is_training = mode == ModeKeys.TRAIN
     # The Keras training.Model adds a name_scope with the name of the model
     # which modifies the constructed graph. Hence we add another name_scope
     # here which is the one before the training.Model one was applied.
@@ -212,6 +269,129 @@ class _DNNModel(training.Model):
     setattr(self, layer_name, layer)
 
 
+def _name_from_scope_name(name):
+  """Returns the name of an op given the name of its scope.
+
+  Args:
+    name: the name of the scope.
+
+  Returns:
+    the name of the op (equal to scope name minus any trailing slash).
+  """
+  return name[:-1] if (name and name[-1] == '/') else name
+
+
+class _DNNModelV2(training.Model):
+  """A DNN Model."""
+
+  def __init__(self,
+               units,
+               hidden_units,
+               feature_columns,
+               activation_fn,
+               dropout,
+               batch_norm,
+               name=None,
+               **kwargs):
+    super(_DNNModelV2, self).__init__(name=name, **kwargs)
+
+    with ops.name_scope(
+        'input_from_feature_columns') as input_feature_column_scope:
+      layer_name = input_feature_column_scope + 'input_layer'
+      if feature_column_lib.is_feature_column_v2(feature_columns):
+        self._input_layer = dense_features_v2.DenseFeatures(
+            feature_columns=feature_columns, name=layer_name)
+      else:
+        raise ValueError(
+            'Received a feature column from TensorFlow v1, but this is a '
+            'TensorFlow v2 Estimator. Please either use v2 feature columns '
+            '(accessible via tf.feature_column.* in TF 2.x) with this '
+            'Estimator, or switch to a v1 Estimator for use with v1 feature '
+            'columns (accessible via tf.compat.v1.estimator.* and '
+            'tf.compat.v1.feature_column.*, respectively.')
+
+    self._dropout = dropout
+    self._batch_norm = batch_norm
+
+    self._hidden_layers = []
+    self._dropout_layers = []
+    self._batch_norm_layers = []
+    self._hidden_layer_scope_names = []
+    for layer_id, num_hidden_units in enumerate(hidden_units):
+      with ops.name_scope('hiddenlayer_%d' % layer_id) as hidden_layer_scope:
+        # Get scope name without the trailing slash.
+        hidden_shared_name = _name_from_scope_name(hidden_layer_scope)
+        hidden_layer = keras_core.Dense(
+            units=num_hidden_units,
+            activation=activation_fn,
+            kernel_initializer=init_ops.glorot_uniform_initializer(),
+            name=hidden_shared_name)
+        self._hidden_layer_scope_names.append(hidden_shared_name)
+        self._hidden_layers.append(hidden_layer)
+        if self._dropout is not None:
+          dropout_layer = keras_core.Dropout(rate=self._dropout)
+          self._dropout_layers.append(dropout_layer)
+        if self._batch_norm:
+          batch_norm_name = hidden_shared_name + '/batchnorm_%d' % layer_id
+          batch_norm_layer = keras_norm.BatchNormalization(
+              # The default momentum 0.99 actually crashes on certain
+              # problem, so here we use 0.999, which is the default of
+              # tf.contrib.layers.batch_norm.
+              momentum=0.999,
+              trainable=True,
+              name=batch_norm_name)
+          self._batch_norm_layers.append(batch_norm_layer)
+
+    with ops.name_scope('logits') as logits_scope:
+      logits_shared_name = _name_from_scope_name(logits_scope)
+      self._logits_layer = keras_core.Dense(
+          units=units,
+          activation=None,
+          kernel_initializer=init_ops.glorot_uniform_initializer(),
+          name=logits_shared_name)
+      self._logits_scope_name = logits_shared_name
+
+  def call(self, features, mode):
+    is_training = mode == ModeKeys.TRAIN
+    net = self._input_layer(features)
+    for i in range(len(self._hidden_layers)):
+      net = self._hidden_layers[i](net)
+      if self._dropout is not None and is_training:
+        net = self._dropout_layers[i](net, training=True)
+      if self._batch_norm:
+        net = self._batch_norm_layers[i](net, training=is_training)
+      _add_hidden_layer_summary(net, self._hidden_layer_scope_names[i])
+
+    logits = self._logits_layer(net)
+    _add_hidden_layer_summary(logits, self._logits_scope_name)
+    return logits
+
+
+def _validate_features(features):
+  if not isinstance(features, dict):
+    raise ValueError('features should be a dictionary of `Tensor`s. '
+                     'Given type: {}'.format(type(features)))
+
+
+def _get_dnn_estimator_spec(use_tpu, head, features, labels, mode, logits,
+                            optimizer):
+  """Get EstimatorSpec for DNN Model."""
+  if use_tpu:
+    return head._create_tpu_estimator_spec(  # pylint: disable=protected-access
+        features=features,
+        mode=mode,
+        labels=labels,
+        optimizer=optimizer,
+        logits=logits)
+  else:
+    return head.create_estimator_spec(
+        features=features,
+        mode=mode,
+        labels=labels,
+        optimizer=optimizer,
+        logits=logits)
+
+
 def _dnn_model_fn(features,
                   labels,
                   mode,
@@ -225,14 +405,14 @@ def _dnn_model_fn(features,
                   config=None,
                   use_tpu=False,
                   batch_norm=False):
-  """Deep Neural Net model_fn.
+  """Deep Neural Net model_fn v1.
 
   Args:
     features: dict of `Tensor`.
-    labels: `Tensor` of shape [batch_size, 1] or [batch_size] labels of
-      dtype `int32` or `int64` in the range `[0, n_classes)`.
-    mode: Defines whether this is training, evaluation or prediction.
-      See `ModeKeys`.
+    labels: `Tensor` of shape [batch_size, 1] or [batch_size] labels of dtype
+      `int32` or `int64` in the range `[0, n_classes)`.
+    mode: Defines whether this is training, evaluation or prediction. See
+      `ModeKeys`.
     head: A `head_lib._Head` instance.
     hidden_units: Iterable of integer number of hidden units per layer.
     feature_columns: Iterable of `feature_column._FeatureColumn` model inputs.
@@ -242,8 +422,8 @@ def _dnn_model_fn(features,
     activation_fn: Activation function applied to each layer.
     dropout: When not `None`, the probability we will drop out a given
       coordinate.
-    input_layer_partitioner: Partitioner for input layer. Defaults
-      to `min_max_variable_partitioner` with `min_slice_size` 64 << 20.
+    input_layer_partitioner: Partitioner for input layer. Defaults to
+      `min_max_variable_partitioner` with `min_slice_size` 64 << 20.
     config: `RunConfig` object to configure the runtime settings.
     use_tpu: Whether to make a DNN model able to run on TPU. Will make function
       return a `_TPUEstimatorSpec` instance and disable variable partitioning.
@@ -255,26 +435,22 @@ def _dnn_model_fn(features,
   Raises:
     ValueError: If features has the wrong type.
   """
-  if not isinstance(features, dict):
-    raise ValueError('features should be a dictionary of `Tensor`s. '
-                     'Given type: {}'.format(type(features)))
 
   optimizer = optimizers.get_optimizer_instance(
       optimizer, learning_rate=_LEARNING_RATE)
+
+  _validate_features(features)
+
   num_ps_replicas = config.num_ps_replicas if config else 0
 
   partitioner = (None if use_tpu else
                  partitioned_variables.min_max_variable_partitioner(
                      max_partitions=num_ps_replicas))
   with variable_scope.variable_scope(
-      'dnn',
-      values=tuple(six.itervalues(features)),
-      partitioner=partitioner):
+      'dnn', values=tuple(six.itervalues(features)), partitioner=partitioner):
     input_layer_partitioner = input_layer_partitioner or (
-        None if use_tpu else
-        partitioned_variables.min_max_variable_partitioner(
-            max_partitions=num_ps_replicas,
-            min_slice_size=64 << 20))
+        None if use_tpu else partitioned_variables.min_max_variable_partitioner(
+            max_partitions=num_ps_replicas, min_slice_size=64 << 20))
 
     logit_fn = dnn_logit_fn_builder(
         units=head.logits_dimension,
@@ -286,20 +462,137 @@ def _dnn_model_fn(features,
         batch_norm=batch_norm)
     logits = logit_fn(features=features, mode=mode)
 
-    if use_tpu:
-      return head._create_tpu_estimator_spec(  # pylint: disable=protected-access
-          features=features,
-          mode=mode,
-          labels=labels,
-          optimizer=optimizer,
-          logits=logits)
-    else:
-      return head.create_estimator_spec(
-          features=features,
-          mode=mode,
-          labels=labels,
-          optimizer=optimizer,
-          logits=logits)
+    return _get_dnn_estimator_spec(use_tpu, head, features, labels, mode,
+                                   logits, optimizer)
+
+
+def _dnn_model_fn_builder_v2(units, hidden_units, feature_columns,
+                             activation_fn, dropout, batch_norm,
+                             features, mode):
+  """Function builder for dnn logits, trainable variables and update ops.
+
+  Args:
+    units: An int indicating the dimension of the logit layer.  In the MultiHead
+      case, this should be the sum of all component Heads' logit dimensions.
+    hidden_units: Iterable of integer number of hidden units per layer.
+    feature_columns: Iterable of `feature_column._FeatureColumn` model inputs.
+    activation_fn: Activation function applied to each layer.
+    dropout: When not `None`, the probability we will drop out a given
+      coordinate.
+    batch_norm: Whether to use batch normalization after each hidden layer.
+    features: This is the first item returned from the `input_fn` passed to
+      `train`, `evaluate`, and `predict`. This should be a single `Tensor` or
+      `dict` of same.
+    mode: Optional. Specifies if this training, evaluation or prediction. See
+      `ModeKeys`.
+
+  Returns:
+    A `Tensor` representing the logits, or a list of `Tensor`'s representing
+      multiple logits in the MultiHead case.
+    A list of trainable variables.
+    A list of update ops.
+
+  Raises:
+    ValueError: If units is not an int.
+  """
+  if not isinstance(units, six.integer_types):
+    raise ValueError('units must be an int.  Given type: {}'.format(
+        type(units)))
+  dnn_model = _DNNModelV2(
+      units,
+      hidden_units,
+      feature_columns,
+      activation_fn,
+      dropout,
+      batch_norm,
+      name='dnn')
+  logits = dnn_model(features, mode)
+  trainable_variables = dnn_model.trainable_variables
+  update_ops = dnn_model.updates
+
+  return logits, trainable_variables, update_ops
+
+
+def dnn_model_fn_v2(features,
+                    labels,
+                    mode,
+                    head,
+                    hidden_units,
+                    feature_columns,
+                    optimizer='Adagrad',
+                    activation_fn=nn.relu,
+                    dropout=None,
+                    config=None,
+                    use_tpu=False,
+                    batch_norm=False):
+  """Deep Neural Net model_fn v2.
+
+  This function is different than _dnn_model_fn_v1 in the way it handles the
+  optimizer when a String optimizer name is passed.
+
+  Args:
+    features: dict of `Tensor`.
+    labels: `Tensor` of shape [batch_size, 1] or [batch_size] labels of dtype
+      `int32` or `int64` in the range `[0, n_classes)`.
+    mode: Defines whether this is training, evaluation or prediction. See
+      `ModeKeys`.
+    head: A `base_head.Head` instance.
+    hidden_units: Iterable of integer number of hidden units per layer.
+    feature_columns: Iterable of `feature_column._FeatureColumn` model inputs.
+    optimizer: String, `tf.keras.optimizers.Optimizer` object, or callable that
+      creates the optimizer to use for training. If not specified, will use the
+      Adagrad optimizer. If it is String, the default learning rate of the
+      optimizer will be used. If it is String, and optimizer does not have a
+      default learning rate, then, a fixed learning rate of 0.05 is used.
+    activation_fn: Activation function applied to each layer.
+    dropout: When not `None`, the probability we will drop out a given
+      coordinate.
+    config: `RunConfig` object to configure the runtime settings.
+    use_tpu: Whether to make a DNN model able to run on TPU. Will make function
+      return a `_TPUEstimatorSpec` instance and disable variable partitioning.
+    batch_norm: Whether to use batch normalization after each hidden layer.
+
+  Returns:
+    An `EstimatorSpec` instance.
+
+  Raises:
+    ValueError: If features has the wrong type.
+  """
+  _validate_features(features)
+
+  del config
+
+  logits, trainable_variables, update_ops = _dnn_model_fn_builder_v2(
+      units=head.logits_dimension,
+      hidden_units=hidden_units,
+      feature_columns=feature_columns,
+      activation_fn=activation_fn,
+      dropout=dropout,
+      batch_norm=batch_norm,
+      features=features,
+      mode=mode)
+
+  # In TRAIN mode, create optimizer and assign global_step variable to
+  # optimizer.iterations to make global_step increased correctly, as Hooks
+  # relies on global step as step counter.
+  if mode == ModeKeys.TRAIN:
+    optimizer = optimizers.get_optimizer_instance_v2(optimizer)
+    optimizer.iterations = training_util.get_or_create_global_step()
+
+  # Create EstimatorSpec.
+  if use_tpu:
+    estimator_spec_fn = head._create_tpu_estimator_spec  # pylint: disable=protected-access
+  else:
+    estimator_spec_fn = head.create_estimator_spec  # pylint: disable=protected-access
+
+  return estimator_spec_fn(
+      features=features,
+      mode=mode,
+      labels=labels,
+      optimizer=optimizer,
+      logits=logits,
+      trainable_variables=trainable_variables,
+      update_ops=update_ops)
 
 
 @estimator_export('estimator.DNNClassifier', v1=[])
@@ -400,10 +693,9 @@ class DNNClassifierV2(estimator.EstimatorV2):
       optimizer='Adagrad',
       activation_fn=nn.relu,
       dropout=None,
-      input_layer_partitioner=None,
       config=None,
       warm_start_from=None,
-      loss_reduction=losses.Reduction.SUM_OVER_BATCH_SIZE,
+      loss_reduction=losses_utils.ReductionV2.SUM_OVER_BATCH_SIZE,
       batch_norm=False,
   ):
     """Initializes a `DNNClassifier` instance.
@@ -441,8 +733,6 @@ class DNNClassifierV2(estimator.EstimatorV2):
         use `tf.nn.relu`.
       dropout: When not `None`, the probability we will drop out a given
         coordinate.
-      input_layer_partitioner: Optional. Partitioner for input layer. Defaults
-        to `min_max_variable_partitioner` with `min_slice_size` 64 << 20.
       config: `RunConfig` object to configure the runtime settings.
       warm_start_from: A string filepath to a checkpoint to warm-start from, or
         a `WarmStartSettings` object to fully configure warm-starting.  If the
@@ -459,8 +749,8 @@ class DNNClassifierV2(estimator.EstimatorV2):
         loss_reduction=loss_reduction)
 
     def _model_fn(features, labels, mode, config):
-      """Call the defined shared _dnn_model_fn."""
-      return _dnn_model_fn(
+      """Call the defined shared dnn_model_fn_v2."""
+      return dnn_model_fn_v2(
           features=features,
           labels=labels,
           mode=mode,
@@ -470,7 +760,6 @@ class DNNClassifierV2(estimator.EstimatorV2):
           optimizer=optimizer,
           activation_fn=activation_fn,
           dropout=dropout,
-          input_layer_partitioner=input_layer_partitioner,
           config=config,
           batch_norm=batch_norm)
 
@@ -506,7 +795,7 @@ class DNNClassifier(estimator.Estimator):
         n_classes, weight_column, label_vocabulary, loss_reduction)
 
     def _model_fn(features, labels, mode, config):
-      """Call the defined shared _dnn_model_fn."""
+      """Call the defined shared dnn_model_fn."""
       return _dnn_model_fn(
           features=features,
           labels=labels,
@@ -629,7 +918,6 @@ class DNNEstimatorV2(estimator.EstimatorV2):
                optimizer='Adagrad',
                activation_fn=nn.relu,
                dropout=None,
-               input_layer_partitioner=None,
                config=None,
                warm_start_from=None,
                batch_norm=False):
@@ -654,8 +942,6 @@ class DNNEstimatorV2(estimator.EstimatorV2):
         use `tf.nn.relu`.
       dropout: When not `None`, the probability we will drop out a given
         coordinate.
-      input_layer_partitioner: Optional. Partitioner for input layer. Defaults
-        to `min_max_variable_partitioner` with `min_slice_size` 64 << 20.
       config: `RunConfig` object to configure the runtime settings.
       warm_start_from: A string filepath to a checkpoint to warm-start from, or
         a `WarmStartSettings` object to fully configure warm-starting.  If the
@@ -665,8 +951,8 @@ class DNNEstimatorV2(estimator.EstimatorV2):
       batch_norm: Whether to use batch normalization after each hidden layer.
     """
     def _model_fn(features, labels, mode, config):
-      """Call the defined shared _dnn_model_fn."""
-      return _dnn_model_fn(
+      """Call the defined shared dnn_model_fn_v2."""
+      return dnn_model_fn_v2(
           features=features,
           labels=labels,
           mode=mode,
@@ -676,7 +962,6 @@ class DNNEstimatorV2(estimator.EstimatorV2):
           optimizer=optimizer,
           activation_fn=activation_fn,
           dropout=dropout,
-          input_layer_partitioner=input_layer_partitioner,
           config=config,
           batch_norm=batch_norm)
     super(DNNEstimatorV2, self).__init__(
@@ -817,10 +1102,9 @@ class DNNRegressorV2(estimator.EstimatorV2):
       optimizer='Adagrad',
       activation_fn=nn.relu,
       dropout=None,
-      input_layer_partitioner=None,
       config=None,
       warm_start_from=None,
-      loss_reduction=losses.Reduction.SUM_OVER_BATCH_SIZE,
+      loss_reduction=losses_utils.ReductionV2.SUM_OVER_BATCH_SIZE,
       batch_norm=False,
   ):
     """Initializes a `DNNRegressor` instance.
@@ -845,15 +1129,13 @@ class DNNRegressorV2(estimator.EstimatorV2):
         used as a key to fetch weight tensor from the `features`. If it is a
         `_NumericColumn`, raw tensor is fetched by key `weight_column.key`,
         then weight_column.normalizer_fn is applied on it to get weight tensor.
-      optimizer: An instance of `tf.Optimizer` used to train the model. Can also
-        be a string (one of 'Adagrad', 'Adam', 'Ftrl', 'RMSProp', 'SGD'), or
-        callable. Defaults to Adagrad optimizer.
+      optimizer: An instance of `tf.keras.optimizers.Optimizer` used to train
+        the model. Can also be a string (one of 'Adagrad', 'Adam', 'Ftrl',
+        'RMSProp', 'SGD'), or callable. Defaults to Adagrad optimizer.
       activation_fn: Activation function applied to each layer. If `None`, will
         use `tf.nn.relu`.
       dropout: When not `None`, the probability we will drop out a given
         coordinate.
-      input_layer_partitioner: Optional. Partitioner for input layer. Defaults
-        to `min_max_variable_partitioner` with `min_slice_size` 64 << 20.
       config: `RunConfig` object to configure the runtime settings.
       warm_start_from: A string filepath to a checkpoint to warm-start from, or
         a `WarmStartSettings` object to fully configure warm-starting.  If the
@@ -869,8 +1151,8 @@ class DNNRegressorV2(estimator.EstimatorV2):
         weight_column=weight_column,
         loss_reduction=loss_reduction)
     def _model_fn(features, labels, mode, config):
-      """Call the defined shared _dnn_model_fn."""
-      return _dnn_model_fn(
+      """Call the defined shared dnn_model_fn_v2."""
+      return dnn_model_fn_v2(
           features=features,
           labels=labels,
           mode=mode,
@@ -880,7 +1162,6 @@ class DNNRegressorV2(estimator.EstimatorV2):
           optimizer=optimizer,
           activation_fn=activation_fn,
           dropout=dropout,
-          input_layer_partitioner=input_layer_partitioner,
           config=config,
           batch_norm=batch_norm)
 
