@@ -541,9 +541,9 @@ class _SDCAModel(object):
               (flat_ids - extras) // ids_per_partition)
 
           # Emulate a conditional using a boolean indicator tensor
-          new_ids = array_ops.where(p_assignments < extras,
-                                    flat_ids % (ids_per_partition + 1),
-                                    (flat_ids - extras) % ids_per_partition)
+          new_ids = array_ops.where_v2(p_assignments < extras,
+                                       flat_ids % (ids_per_partition + 1),
+                                       (flat_ids - extras) % ids_per_partition)
 
           # Cast partition assignments to int32 for use in dynamic_partition.
           # There really should not be more than 2^32 partitions.
@@ -640,7 +640,7 @@ class _SDCAModel(object):
               update_ops.append(state_ops.assign_add(v, split_update))
           else:
             update_ops.append(state_ops.assign_add(w, u))
-      if not global_step:
+      if global_step is None:
         return control_flow_ops.group(*update_ops)
       with ops.control_dependencies(update_ops):
         return state_ops.assign_add(global_step, 1, name=name).op
