@@ -25,18 +25,22 @@ from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import sparse_tensor
 from tensorflow.python.keras import metrics
+from tensorflow.python.keras.utils import losses_utils
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import lookup_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import sparse_ops
 from tensorflow.python.ops.losses import losses
+from tensorflow.python.util.tf_export import estimator_export
 from tensorflow_estimator.python.estimator import model_fn
 from tensorflow_estimator.python.estimator.canned import metric_keys
 from tensorflow_estimator.python.estimator.canned import prediction_keys
 from tensorflow_estimator.python.estimator.export import export_output
 from tensorflow_estimator.python.estimator.head import base_head
+from tensorflow_estimator.python.estimator.mode_keys import ModeKeys
 
 
+@estimator_export('estimator.MultiLabelHead')
 class MultiLabelHead(base_head.Head):
   """Creates a `Head` for multi-label classification.
 
@@ -91,7 +95,7 @@ class MultiLabelHead(base_head.Head):
         features=features,
         mode=mode,
         labels=labels,
-        optimizer=tf.AdagradOptimizer(learning_rate=0.1),
+        optimizer=tf.keras.optimizers.Adagrad(lr=0.1),
         logits=logits)
 
   my_estimator = tf.estimator.Estimator(model_fn=_my_model_fn)
@@ -122,7 +126,7 @@ class MultiLabelHead(base_head.Head):
       names for which per-class metrics are evaluated. If integers, all must be
       in the range `[0, n_classes - 1]`. If strings, all must be in
       `label_vocabulary`.
-    name: name of the head. If provided, summary and metrics keys will be
+    name: Name of the head. If provided, summary and metrics keys will be
       suffixed by `"/" + name`. Also used as `name_scope` when creating ops.
   """
 
@@ -131,7 +135,7 @@ class MultiLabelHead(base_head.Head):
                weight_column=None,
                thresholds=None,
                label_vocabulary=None,
-               loss_reduction=losses.Reduction.SUM_OVER_BATCH_SIZE,
+               loss_reduction=losses_utils.ReductionV2.SUM_OVER_BATCH_SIZE,
                loss_fn=None,
                classes_for_class_based_metrics=None,
                name=None):
@@ -156,9 +160,7 @@ class MultiLabelHead(base_head.Head):
 
     if loss_fn:
       base_head.validate_loss_fn_args(loss_fn)
-    if (loss_reduction not in losses.Reduction.all() or
-        loss_reduction == losses.Reduction.NONE):
-      raise ValueError('Invalid loss_reduction: {}'.format(loss_reduction))
+    base_head.validate_loss_reduction(loss_reduction)
     if classes_for_class_based_metrics:
       classes_for_class_based_metrics = tuple(classes_for_class_based_metrics)
       if isinstance(classes_for_class_based_metrics[0], six.string_types):
@@ -243,7 +245,7 @@ class MultiLabelHead(base_head.Head):
   # An attribute for lookup table. Note that for Graph execution, the lookup
   # table is created on demand to make sure the lookup table is in the same
   # graph as its input tensors for `train` and `eval` of Estimator (as Estimator
-  # re-creates graphes for `train`, `eval` and `predict`).
+  # re-creates graphs for `train`, `eval` and `predict`).
   _cached_class_id_table = None
 
   @property
@@ -313,7 +315,8 @@ class MultiLabelHead(base_head.Head):
           expected_loss_dim=1)
     else:
       unweighted_loss = losses.sigmoid_cross_entropy(
-          multi_class_labels=processed_labels, logits=logits,
+          multi_class_labels=processed_labels,
+          logits=logits,
           reduction=losses.Reduction.NONE)
       # Averages loss over classes.
       unweighted_loss = math_ops.reduce_mean(
@@ -324,7 +327,7 @@ class MultiLabelHead(base_head.Head):
         logits=logits)
     return unweighted_loss, weights
 
-  def loss(self, logits, labels, features=None, mode=None,
+  def loss(self, labels, logits, features=None, mode=None,
            regularization_losses=None):
     """Returns regularized training loss. See `base_head.Head` for details."""
     del mode  # Unused for this head.
@@ -334,8 +337,10 @@ class MultiLabelHead(base_head.Head):
       processed_labels = self._processed_labels(logits, labels)
       unweighted_loss, weights = self._unweighted_loss_and_weights(
           logits, processed_labels, features)
-      training_loss = losses.compute_weighted_loss(
-          unweighted_loss, weights=weights, reduction=self._loss_reduction)
+      training_loss = losses_utils.compute_weighted_loss(
+          unweighted_loss,
+          sample_weight=weights,
+          reduction=self._loss_reduction)
       regularization_loss = math_ops.add_n(
           regularization_losses) if regularization_losses is not None else None
       regularized_training_loss = (
@@ -379,9 +384,9 @@ class MultiLabelHead(base_head.Head):
       # Mean metric.
       eval_metrics = {}
       eval_metrics[self._loss_mean_key] = metrics.Mean(name=keys.LOSS_MEAN)
-      # TODO(b/118843532): create Keras metrics
-      # eval_metrics[self._auc] = metrics.Precision(name=keys.AUC)
-      # eval_metrics[self._auc_pr_key] = metrics.Precision(name=keys.AUC_PR)
+      # The default summation_method is "interpolation" in the AUC metric.
+      eval_metrics[self._auc_key] = metrics.AUC(name=keys.AUC)
+      eval_metrics[self._auc_pr_key] = metrics.AUC(curve='PR', name=keys.AUC_PR)
       if regularization_losses is not None:
         eval_metrics[self._loss_regularization_key] = metrics.Mean(
             name=keys.LOSS_REGULARIZATION)
@@ -394,12 +399,10 @@ class MultiLabelHead(base_head.Head):
         eval_metrics[self._recall_keys[i]] = metrics.Recall(
             name=self._recall_keys[i], thresholds=threshold)
       for i in range(len(self._classes_for_class_based_metrics)):
-        # TODO(b/118843532): create Keras metrics
         eval_metrics[self._prob_keys[i]] = metrics.Mean(name=self._prob_keys[i])
-        # eval_metrics[self._auc_keys[i]] = metrics.AUC(name=self._auc_keys[i])
-        # eval_metrics[self._auc_pr_keys[i]] = metrics.AUC_PR(
-        #     name=self._auc_pr_keys[i])
-
+        eval_metrics[self._auc_keys[i]] = metrics.AUC(name=self._auc_keys[i])
+        eval_metrics[self._auc_pr_keys[i]] = metrics.AUC(
+            curve='PR', name=self._auc_pr_keys[i])
     return eval_metrics
 
   def update_metrics(self, eval_metrics, features, logits, labels,
@@ -416,20 +419,21 @@ class MultiLabelHead(base_head.Head):
     # Update metrics.
     eval_metrics[self._loss_mean_key].update_state(
         values=unweighted_loss, sample_weight=weights)
-    # TODO(b/118843532): update Keras metrics
-    # eval_metrics[self._auc_key].update_state(...)
-    # eval_metrics[self._auc_pr_key].update_state(...)
+    eval_metrics[self._auc_key].update_state(
+        y_true=processed_labels, y_pred=probabilities, sample_weight=weights)
+    eval_metrics[self._auc_pr_key].update_state(
+        y_true=processed_labels, y_pred=probabilities, sample_weight=weights)
     if regularization_losses is not None:
       regularization_loss = math_ops.add_n(regularization_losses)
       eval_metrics[self._loss_regularization_key].update_state(
           values=regularization_loss)
     for i in range(len(self._thresholds)):
       eval_metrics[self._accuracy_keys[i]].update_state(
-          y_true=labels, y_pred=probabilities, sample_weight=weights)
+          y_true=processed_labels, y_pred=probabilities, sample_weight=weights)
       eval_metrics[self._precision_keys[i]].update_state(
-          y_true=labels, y_pred=probabilities, sample_weight=weights)
+          y_true=processed_labels, y_pred=probabilities, sample_weight=weights)
       eval_metrics[self._recall_keys[i]].update_state(
-          y_true=labels, y_pred=probabilities, sample_weight=weights)
+          y_true=processed_labels, y_pred=probabilities, sample_weight=weights)
     for i, class_id in enumerate(self._classes_for_class_based_metrics):
       batch_rank = array_ops.rank(probabilities) - 1
       begin = array_ops.concat(
@@ -440,17 +444,21 @@ class MultiLabelHead(base_head.Head):
           axis=0)
       class_probabilities = array_ops.slice(
           probabilities, begin=begin, size=size)
-      # class_labels = array_ops.slice(labels, begin=begin, size=size)
-      # TODO(b/118843532): update Keras metrics
+      class_labels = array_ops.slice(processed_labels, begin=begin, size=size)
       base_head.update_metric_with_broadcast_weights(
           eval_metrics[self._prob_keys[i]], class_probabilities, weights)
-      # eval_metrics[self._auc_keys[i]].update_state(...)
-      # eval_metrics[self._auc_pr_key[i]].update_state(...)
+      eval_metrics[self._auc_keys[i]].update_state(
+          y_true=class_labels, y_pred=class_probabilities,
+          sample_weight=weights)
+      eval_metrics[self._auc_pr_keys[i]].update_state(
+          y_true=class_labels, y_pred=class_probabilities,
+          sample_weight=weights)
     return eval_metrics
 
   def _create_tpu_estimator_spec(
       self, features, mode, logits, labels=None, optimizer=None,
-      train_op_fn=None, regularization_losses=None):
+      trainable_variables=None, train_op_fn=None, update_ops=None,
+      regularization_losses=None):
     """Returns an `model_fn._TPUEstimatorSpec`.
 
     Args:
@@ -462,16 +470,28 @@ class MultiLabelHead(base_head.Head):
         with shape `[D0, D1, ... DN, n_classes]` or `SparseTensor` with
         `dense_shape` `[D0, D1, ... DN, ?]`. `labels` is required argument when
         `mode` equals `TRAIN` or `EVAL`.
-      optimizer: `Optimizer` instance to optimize the loss in TRAIN mode.
-        Namely, sets `train_op = optimizer.minimize(loss, global_step)`, which
-        updates variables and increments `global_step`.
+      optimizer: An `tf.keras.optimizers.Optimizer` instance to optimize the
+         loss in TRAIN mode. Namely, sets
+        `train_op = optimizer.get_updates(loss, trainable_variables)`,
+        which updates variables to minimize `loss`.able_variables)`,
+        which updates variables to minimize `loss`.
+      trainable_variables: A list or tuple of `Variable` objects to update to
+        minimize `loss`. In Tensorflow 1.x, by default these are the list of
+        variables collected in the graph under the key
+        `GraphKeys.TRAINABLE_VARIABLES`. As Tensorflow 2.x doesn't have
+        collections and GraphKeys, trainable_variables need to be passed
+        explicitly here.
       train_op_fn: Function that takes a scalar loss `Tensor` and returns
         `train_op`. Used if `optimizer` is `None`.
+      update_ops: A list or tuple of update ops to be run at training time. For
+        example, layers such as BatchNormalization create mean and variance
+        update ops that need to be run at training time. In Tensorflow 1.x,
+        these are thrown into an UPDATE_OPS collection. As Tensorflow 2.x
+        doesn't have collections, update_ops need to be passed explicitly here.
       regularization_losses: A list of additional scalar losses to be added to
         the training loss, such as regularization losses. These losses are
         usually expressed as a batch average, so for best results users need to
-        set `loss_reduction=SUM_OVER_BATCH_SIZE` or
-        `loss_reduction=SUM_OVER_NONZERO_WEIGHTS` when creating the head to
+        set `loss_reduction=SUM_OVER_BATCH_SIZE` when creating the head to
         avoid scaling errors.
     Returns:
       `model_fn._TPUEstimatorSpec`.
@@ -483,13 +503,13 @@ class MultiLabelHead(base_head.Head):
       # Predict.
       pred_keys = prediction_keys.PredictionKeys
       predictions = self.predictions(logits)
-      if mode == model_fn.ModeKeys.PREDICT:
+      if mode == ModeKeys.PREDICT:
         probabilities = predictions[pred_keys.PROBABILITIES]
         classifier_output = base_head.classification_output(
             scores=probabilities, n_classes=self._n_classes,
             label_vocabulary=self._label_vocabulary)
         return model_fn._TPUEstimatorSpec(  # pylint:disable=protected-access
-            mode=model_fn.ModeKeys.PREDICT,
+            mode=ModeKeys.PREDICT,
             predictions=predictions,
             export_outputs={
                 base_head.DEFAULT_SERVING_KEY: classifier_output,
@@ -502,10 +522,10 @@ class MultiLabelHead(base_head.Head):
           logits=logits, labels=labels, features=features, mode=mode,
           regularization_losses=regularization_losses)
       # Eval.
-      if mode == model_fn.ModeKeys.EVAL:
+      if mode == ModeKeys.EVAL:
         eval_metrics = self.metrics(regularization_losses=regularization_losses)
         return model_fn._TPUEstimatorSpec(  # pylint:disable=protected-access
-            mode=model_fn.ModeKeys.EVAL,
+            mode=ModeKeys.EVAL,
             predictions=predictions,
             loss=regularized_training_loss,
             eval_metrics=base_head.create_eval_metrics_tuple(
@@ -518,12 +538,20 @@ class MultiLabelHead(base_head.Head):
                 }))
       # Train.
       train_op = base_head.create_estimator_spec_train_op(
-          self._name, optimizer, train_op_fn, regularized_training_loss)
+          head_name=self._name,
+          optimizer=optimizer,
+          train_op_fn=train_op_fn,
+          update_ops=update_ops,
+          trainable_variables=trainable_variables,
+          regularized_training_loss=regularized_training_loss,
+          loss_reduction=self._loss_reduction)
     # Create summary.
     base_head.create_estimator_spec_summary(
-        regularized_training_loss, regularization_losses, self._summary_key)
+        regularized_training_loss=regularized_training_loss,
+        regularization_losses=regularization_losses,
+        summary_key_fn=self._summary_key)
     return model_fn._TPUEstimatorSpec(  # pylint: disable=protected-access
-        mode=model_fn.ModeKeys.TRAIN,
+        mode=ModeKeys.TRAIN,
         predictions=predictions,
         loss=regularized_training_loss,
         train_op=train_op)
